﻿/*

MetaTrader Data Export
mtde.h
v1.0.0.0
2021-08-28
Copyright © 2021 Heinz Traub
www.tripleheinz.com

*/

/* MetaTrader Data Export.
A two-component (Expert Advisor + DLL) solution for exporting tick and current period data for the current symbol. */

#include <stdint.h>
#include <stdbool.h>

// This has been implemented for convenience in case someone wants to use the DLL externally (outside the MT terminal) so that the imports are declared.
#ifdef MTDE_DLL_COMPILE
	#define MTDE_DLL __declspec(dllexport)
#else
	#define MTDE_DLL __declspec(dllimport)
#endif

typedef uint32_t MTDE_ReturnCode;
typedef uint64_t MTDE_Handle;

const MTDE_ReturnCode MTDE_SUCCESS = 0;

#pragma pack(1) // No alignment is performed. Members are packed together to match the behaviour in MQL5.

struct MTDE_InitData
{
	wchar_t symbol[50];
	int32_t period;
	uint32_t period_seconds;
	wchar_t period_string[25];
	uint32_t precision;
	bool export_tick_data;
	bool export_bar_data;
};

struct MTDE_MqlTick
{
	uint64_t time;
	double bid;
	double ask;
	double last;
	uint64_t volume;
	int64_t time_msc;
	uint32_t flags;
	double volume_real;
};

struct MTDE_TickData
{
	MTDE_Handle handle;
	struct MTDE_MqlTick tick;
};

struct MTDE_MqlRates
{
	uint64_t time;
	double open;
	double high;
	double low;
	double close;
	int64_t tick_volume;
	int32_t spread;
	int64_t real_volume;
};

struct MTDE_BarData
{
	MTDE_Handle handle;
	struct MTDE_MqlRates rates;
};

MTDE_DLL MTDE_ReturnCode MTDE_Initialize(const struct MTDE_InitData* data, MTDE_Handle* handle); // Perform initialization routines and return (by parameter) an instance handle associated to arbitrary data.
MTDE_DLL MTDE_ReturnCode MTDE_Deinitialize(const MTDE_Handle handle); // Perform deinitialization routines on the handle.
MTDE_DLL MTDE_ReturnCode MTDE_OnTickData(const struct MTDE_TickData* data); // Process data generated by a tick.
MTDE_DLL MTDE_ReturnCode MTDE_OnBarData(const struct MTDE_BarData* data); // Process data generated by a bar.