﻿/*

MetaTrader Data Export
mtde.c
v1.0.0.0
2021-08-28
Copyright © 2021 Heinz Traub
www.tripleheinz.com

*/

// MTDE Template DLL. Modify code to suit your needs.

#include <mtde.h>
#include <stdlib.h>
#include <wchar.h>

enum MTDE_ERROR_CODES
{
	MTDE_Error_Unknown = 10000,
	// 10001 and 10002 are reserved (shhh, it's a secret).
	MTDE_Error_No_Operation = 10003,
	MTDE_Error_Handle_malloc,
	// Insert your own error codes here.
	// <-----
};

struct MTDE_Instance
{
	wchar_t symbol[50];
	int32_t period;
	uint32_t period_seconds;
	wchar_t period_string[25];
	uint32_t precision;
	bool export_tick_data;
	bool export_bar_data;
	// Insert your own variables here.
	// <-----
};

MTDE_DLL MTDE_ReturnCode MTDE_Initialize(const struct MTDE_InitData* data, MTDE_Handle* handle)
{
	if(!data->export_tick_data && !data->export_bar_data)
		return MTDE_Error_No_Operation;
	
	struct MTDE_Instance* instance = (struct MTDE_Instance*)malloc(sizeof(struct MTDE_Instance));
	if(!instance)
		return MTDE_Error_Handle_malloc;
	
	wcscpy(instance->symbol, data->symbol);
	instance->period = data->period;
	instance->period_seconds = data->period_seconds;
	wcscpy(instance->period_string, data->period_string);
	instance->precision = data->precision;
	instance->export_tick_data = data->export_tick_data;
	instance->export_bar_data = data->export_bar_data;
	
	// Insert your own initialization routines here.
	// <-----
	
	if(instance->export_tick_data)
	{
		// Insert your own TICK data specific initialization routines here.
		// <-----
		
		// Remember that if something fails then you have to release the allocated resources and return an error code.
		/*if("Something bad happened")
		{
			// Release your TICK resources here.
			// <-----
			free(instance);
			return "My error code";
		}*/
	}
	if(instance->export_bar_data)
	{
		// Insert your own BAR data specific initialization routines here.
		// <-----
		
		// Remember that if something fails then you have to release the allocated resources and return an error code.
		/*if("Something bad happened")
		{
			// Release your BAR resources here.
			// <-----
			
			// Remember to also release resources specific to the TICK data initialization.
			if(instance->export_tick_data)
			{
				// Release your TICK resources here.
				// <-----
			}
			free(instance);
			return "My error code";
		}*/
	}
	
	*handle = (MTDE_Handle)instance; // Return handle by parameter.
	return MTDE_SUCCESS;
}

MTDE_DLL MTDE_ReturnCode MTDE_OnTickData(const struct MTDE_TickData* data)
{
	if(data && data->handle)
	{
		struct MTDE_Instance* instance = (struct MTDE_Instance*)data->handle;
		if(instance->export_tick_data)
		{
			// Insert your own TICK data export routines here.
			// <-----
		}
	}
	return MTDE_SUCCESS;
}

MTDE_DLL MTDE_ReturnCode MTDE_OnBarData(const struct MTDE_BarData* data)
{
	if(data && data->handle)
	{
		struct MTDE_Instance* instance = (struct MTDE_Instance*)data->handle;
		if(instance->export_bar_data)
		{
			// Insert your own BAR data export routines here.
			// <-----
		}
	}
	return MTDE_SUCCESS;
}

MTDE_DLL MTDE_ReturnCode MTDE_Deinitialize(const MTDE_Handle handle)
{
	if(handle)
	{
		struct MTDE_Instance* instance = (struct MTDE_Instance*)handle;
		// Insert your own Deinitialization routines here.
		// <-----
		if(instance->export_tick_data)
		{
			// Insert your own TICK data specific Deinitialization routines here.
			// <-----
		}
		if(instance->export_bar_data)
		{
			// Insert your own BAR data specific Deinitialization routines here.
			// <-----
		}
		free(instance);
	}
	return MTDE_SUCCESS;
}